(function (window, document) {
  function VLibrasPlayer() {
    this.loaded      = false;
    this.glosa       = undefined;
    this.canvas      = undefined;
    this.WebPlayer   = undefined;
  }

  var ENDPOINT = 'http://150.165.204.30/glosa';
  var noop = function () {};

  VLibrasPlayer.prototype.translate = function (text) {
    var self = this;
    text = encodeURI(text);

    (self.onTranslateStart || noop)();
    get(ENDPOINT, { texto: text },
      function (status, response) {
        (self.onTranslateEnd || noop)();
        self.play(response || decodeURI(text).toUpperCase());
      });
  };

  VLibrasPlayer.prototype.play = function (glosa) {
    this.glosa = glosa || this.glosa;
    if (this.glosa !== undefined && this.loaded === true) {
      this.sendMessage('PlayerManager', 'catchGlosa', this.glosa);
    }
  };

  VLibrasPlayer.prototype.stop = function () {
    this.sendMessage('PlayerManager', 'StopAnimations');
  };

  VLibrasPlayer.prototype.setSpeed = function (speed) {
    this.sendMessage('PlayerManager', 'setSlider', speed);
  };

  VLibrasPlayer.prototype.switchSubtitle = function () {
    this.sendMessage('PlayerManager', 'switchSubtitleStatus');
  };

  VLibrasPlayer.prototype.addProgressListener = function (callback) {
    this.onProgress = callback;
  };

  VLibrasPlayer.prototype.addLoadListener = function (callback) {
    this.onLoad = callback;
  };

  VLibrasPlayer.prototype.addTranslateStartListener = function (callback) {
    this.onTranslateStart = callback;
  };

  VLibrasPlayer.prototype.addTranslateEndListener = function (callback) {
    this.onTranslateEnd = callback;
  };

  VLibrasPlayer.prototype.addErrorListener = function (callback) {
    this.onError = callback;
  }

  VLibrasPlayer.prototype.load = function () {
    this.loaded = true;
    this.onLoad();
    this.play();
  };

  VLibrasPlayer.prototype.getCanvasElement = function () {
    return this.canvas;
  };

  VLibrasPlayer.prototype.getPath = function () {
    return this.path;
  };

  VLibrasPlayer.prototype.isLoaded = function () {
    return this.loaded;
  }

  VLibrasPlayer.prototype.initialize = function (path, wrapper) {
    this.path = path;

    var script = document.createElement('script');
    var browser = navigator.userAgent.toLowerCase();

    if (/^((?!chrome|android).)*safari/i.test(browser) || !checkWebGLCompability()) {
      this.canvas = document.createElement('div');

      script.src = path + '/Player/WebPlayer/Setup.js';
      var self = this;
      script.onload = function () {
        var params = {
          logoimage: self.path + '/Images/progresslogo.png',
          progressbarimage: self.path + '/Images/fullbar.png',
          progressframeimage: self.path + '/Images/loadingbar.png'
        };

        self.WebPlayer = new UnityObject2(params);
        self.WebPlayer.observeProgress(function (progress) {
          switch(progress.pluginStatus) {
            case "unsupported":
            case "broken":
            case "missing":
              self.onError(progress.pluginStatus);
            break;
          }
        });

        self.WebPlayer.initPlugin(self.canvas, self.path + '/Player/WebPlayer/rnp_webplayer.unity3d');
      };
    } else {
      this.canvas = document.createElement('canvas');
      this.canvas.classList.add('emscripten');

      script.src = path + '/Player/WebGL/Release/Setup.js';
    }

    this.canvas.classList.add('vw-canvas');
    wrapper.appendChild(this.canvas);

    document.body.appendChild(script);
  };

  VLibrasPlayer.prototype.sendMessage = function(object, method, params) {
    if (this.WebPlayer) {
      this.WebPlayer.getUnity().SendMessage(object, method, params);
    } else {
      window.SendMessage(object, method, params);
    }
  };

  window.onLoadPlayer = function () {
    this.VLibrasPlayer.load();
  };

  window.updateProgress = function (progress) {
    (this.VLibrasPlayer.onProgress || noop)(progress);
  };

  window.VLibrasPlayer = new VLibrasPlayer();

  function get(url, params, callback) {
    url += '?';
    for (param in params) {
      url += param + '=' + params[param];
      url += ',';
    }
    url.slice(-1, 1);

    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, true);

    xhr.onreadystatechange = function() {
      if (xhr.readyState != 4) return;
      callback(xhr.status, xhr.responseText);
    };

    xhr.send();
  };

  function checkWebGLCompability() {
    var canvas = document.createElement('canvas');
    var gl = canvas.getContext('webgl');
    if (!gl) {
      gl = canvas.getContext('experimental-webgl');
      if (!gl) return false;
    }

    return true;
  };
})(window, document);